#!/bin/sh

# Definitions:
UBOOT_VAR_RFS_ID=rfsID
UBOOT_VAR_KERNEL_ADDR=kerneladdr
MD5SUM_FILE1=md5sums.txt
MD5SUM_FILE2=md5sum.txt
KERNEL_IMAGE=uImage.B
RFS_IMAGE=rfs.jffs2
POST_UPDATE_SCRIPT=PostUpdateScript.sh
CUSTOMIZING_SCRIPT=customize.sh
VERSION_CURRENT=`cat /version.txt`
VERSION_NEW=`cat version.txt 2>/dev/null`
UPDATE_LOGFILE=/var/log/update.log
UPDATE_HOUR_METER_LOG=/var/log/update-hour-meter.log

# Kernel information:
KERNEL_ADDR1=80000
KERNEL_MTD_NO1=4
KERNEL_ADDR2=380000
KERNEL_MTD_NO2=5
KERNEL_LENGTH=300000

# RFS information:
RFS_ID1=1f06
RFS_ADDR1=680000
RFS_MTD_NO1=6
RFS_ID2=1f09
RFS_ADDR2=4800000
RFS_MTD_NO2=9
RFS_LENGTH=2000000

#
# Script file name.
#
SCRIPT_NAME=$(basename $0)

if [ -f ${MD5SUM_FILE1} ] ; then
    MD5SUM_FILE=${MD5SUM_FILE1}
elif [ -f ${MD5SUM_FILE2} ] ; then
    MD5SUM_FILE=${MD5SUM_FILE2}
else
    # Neither MD5SUM_FILE{1,2} exist -> reject the update:
    exit 1
fi

# Test integrity by checking md5sums:
md5sum -c ${MD5SUM_FILE} >/dev/null 2>&1
if [ "0" -ne "$?" ] ; then
    # Checksum mismatch
    exit 1
fi


#
# Firmware update compatibility check. Abort if not compatible or an error occurred.
# Note: special handling for old firmware version update (no whitelist file available):
#       allow update if 
#           - whitelist file not available in current firmware 
#             AND
#           - current firmware version is lower than a reference firmware version
#
COMP_CHECK_SCRIPT=FirmwareUpdateCompatibilityCheck.sh
if [ -f ${COMP_CHECK_SCRIPT} ] ; then
    /bin/sh ${COMP_CHECK_SCRIPT}
    RET=$?
    if [ "0" -ne "${RET}" ] ; then
        # NOTE: 2 is the error exit code if whitelist file not found in current firmware
        if [ "2" -eq "${RET}" ] ; then
            #
            # special handling updating old firmware versions
            # current firmware whitelist file not available, check version
            #
            REF_VER_MAJOR=3 #reference firmware major number
            REF_VER_MINOR=0 #reference firmware minor number
            CUR_VER_MAJOR=`echo ${VERSION_CURRENT} | awk 'BEGIN {FS="."}{print $1}'`
            CUR_VER_MINOR=`echo ${VERSION_CURRENT} | awk 'BEGIN {FS="."}{print $2}'`
            if [ ${CUR_VER_MAJOR} -lt ${REF_VER_MAJOR} ] ; then
                # current firmware version is lower than reference: allow update
                LOG_STR="Firmware update compatibility check failed but current firmware version"
                LOG_STR="${LOG_STR} is lower than the reference firmware version, update allowed."
                logger -p notice -t ${SCRIPT_NAME} "${LOG_STR}"
                echo "$(date): ${SCRIPT_NAME}: NOTICE: ${LOG_STR}" >>${UPDATE_LOGFILE}
            elif [ ${CUR_VER_MAJOR} -eq ${REF_VER_MAJOR} ] ; then
                if [ ${CUR_VER_MINOR} -lt ${REF_VER_MINOR} ] ; then
                    # current firmware version is lower than reference: allow update
                    LOG_STR="Firmware update compatibility check failed but current firmware version"
                    LOG_STR="${LOG_STR} is lower than the reference firmware version, update allowed."
                    logger -p notice -t ${SCRIPT_NAME} "${LOG_STR}"
                    echo "$(date): ${SCRIPT_NAME}: NOTICE: ${LOG_STR}" >>${UPDATE_LOGFILE}
                else
                    # current firmware version is greater than reference: error (whitelist not found)
                    LOG_STR="The firmware update compatibility check failed (error code: ${RET})."
                    logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
                    echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
                    exit 14
                fi
            else
                # current firmware version is greater than reference: error (whitelist not found)
                LOG_STR="The firmware update compatibility check failed (error code: ${RET})."
                logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
                echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
                exit 14
            fi
        else
            # other error exit code from COMP_CHECK_SCRIPT than 2
            LOG_STR="The firmware update compatibility check failed (error code: ${RET})."
            logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
            echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
            exit 14
        fi
    fi
else
    # script not found, log and exit
    LOG_STR="The firmware update compatibility check script not found (${COMP_CHECK_SCRIPT})."
    logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
    echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
    exit 14
fi


# When files are not available, exit:
test -f ${KERNEL_IMAGE} || exit 2
test -f ${RFS_IMAGE} || exit 2

# Determine new RFS ID and address:
RFS_ID=`fw_getenv ${UBOOT_VAR_RFS_ID} 2>/dev/null`
if [ "x${RFS_ID}" = "x" ] ; then
    # Cannot obtain current setting.
    NEW_RFS_ID=${RFS_ID1}
    NEW_RFS_START_ADDRESS=${RFS_ADDR1}
    NEW_RFS_MTD_NO=${RFS_MTD_NO1}
else
    if [ "${RFS_ID1}" == "${RFS_ID}" ] ; then
        NEW_RFS_ID=${RFS_ID2}
        NEW_RFS_START_ADDRESS=${RFS_ADDR2}
        NEW_RFS_MTD_NO=${RFS_MTD_NO2}
    elif [ "${RFS_ID2}" == "${RFS_ID}" ] ; then
        NEW_RFS_ID=${RFS_ID1}
        NEW_RFS_MTD_NO=${RFS_MTD_NO1}
        NEW_RFS_START_ADDRESS=${RFS_ADDR1}
    else
        exit 3
    fi
fi

# Determine new kernel address:
KERNEL_ADDR=`fw_getenv ${UBOOT_VAR_KERNEL_ADDR} 2>/dev/null`
if [ "x${KERNEL_ADDR}" == "x" ] ; then
    NEW_KERNEL_ADDR=${KERNEL_ADDR1}
    NEW_KERNEL_MTD_NO=${KERNEL_MTD_NO1}
else
    if [ "${KERNEL_ADDR1}" -eq "${KERNEL_ADDR}" ] ; then
        NEW_KERNEL_ADDR=${KERNEL_ADDR2}
        NEW_KERNEL_MTD_NO=${KERNEL_MTD_NO2}
    elif [ "${KERNEL_ADDR2}" -eq "${KERNEL_ADDR}" ] ; then
        NEW_KERNEL_ADDR=${KERNEL_ADDR1}
        NEW_KERNEL_MTD_NO=${KERNEL_MTD_NO1}
    else
        exit 3
    fi
fi

# Check if the board_id variable is already set. If not, we set it to default
SET_BOARD_ID=0
# Patch: 
# We grep for the name of the variable because fw_printenv always returns 0. 
# Even if the variable does not exist.
fw_printenv board_id 2>/dev/null |grep "board_id=" >/dev/null
if [ "0" != "$?" ] ; then
    # not found, add it:
    SET_BOARD_ID=1
fi 

LOGSTR="$(date): Performing a FullUpdate from version ${VERSION_CURRENT} to version ${VERSION_NEW} ..."
echo ${LOGSTR} >>${UPDATE_LOGFILE}
logger ${LOGSTR}

# Perform the flash erasing/writing:
flash_eraseall -j -q /dev/mtd${NEW_KERNEL_MTD_NO} || exit 4
nandwrite -p -q /dev/mtd${NEW_KERNEL_MTD_NO} ${KERNEL_IMAGE} || exit 5


flash_eraseall -j -q /dev/mtd${NEW_RFS_MTD_NO} || exit 6
nandwrite -p -q /dev/mtd${NEW_RFS_MTD_NO} ${RFS_IMAGE} || exit 7


if test ! -d /var/Counter/DeviceSpecific ; then
    # We are dealing with old parameter structure (<1.12), VGA paras may be missing:
    GENERATE_LUT_TOOL=./vgaLutGenerator
    if [ ! -x ${GENERATE_LUT_TOOL} ] ; then
        exit 10
    fi
        ${GENERATE_LUT_TOOL} >/dev/null 2>&1
    RET=$?
    if [ "0" -ne "${RET}" ] ; then
        exit 10
    fi
fi

# update some UBoot environment variables
fw_setenv init_ba5 'setenv ba5 root=$(rfsID) ro rootfstype=jffs2' >/dev/null || exit 9
fw_setenv init_ba6 'setenv ba6 aglaia_versctrl.board_id=$(board_id)' >/dev/null || exit 9
fw_setenv init_ba 'run init_ba1; run init_ba2; run init_ba3; run init_ba4; run init_ba5; run init_ba6' >/dev/null || exit 9
fw_setenv rm_ba 'setenv ba1; setenv ba2; setenv ba3; setenv ba4; setenv ba5; setenv ba6' >/dev/null || exit 9
fw_setenv bc_f 'run init_ba; slew slow; nand read.jffs2 20400000 $(kerneladdr) 300000; setenv bootargs $(ba1) $(ba3) $(ba5) $(ba6); run rm_ba; bootm 20400000' >/dev/null || exit 9
fw_setenv bc_n 'run init_ba; slew slow; tftp 20400000 $(userid)/uImage.B; setenv bootargs $(ba1) $(ba2) $(ba3) $(ba4) $(ba6); run rm_ba; bootm 20400000' >/dev/null

if [ "1" -eq "${SET_BOARD_ID}" ] ; then
    fw_setenv board_id BH3010 >/dev/null || exit 9
fi

CREATE_CONFIG_STRUCTURE_SCRIPT=CreateConfigStructure.sh
if [ -f ${CREATE_CONFIG_STRUCTURE_SCRIPT} ] ; then
    /bin/sh ${CREATE_CONFIG_STRUCTURE_SCRIPT}
    RET=$?
    if [ "0" -ne "$RET" ] ; then
        echo "$(date): Creating config structure failed" >>${UPDATE_LOGFILE}
        exit 11
    fi
fi


# CR 382102: patch (binary) resolution parameter files (AutoFpga value)
PATCH_RESOLUTION_PARAM_FILES_SCRIPT=PatchResolutionParamFiles.sh
if [ -f ${PATCH_RESOLUTION_PARAM_FILES_SCRIPT} ] ; then
    /bin/sh ${PATCH_RESOLUTION_PARAM_FILES_SCRIPT}
    RET=$?
    if [ "0" -ne "$RET" ] ; then
        echo "$(date): Patch resolution parameter files failed" >>${UPDATE_LOGFILE}
        exit 12
    fi
fi


# convert binary parameter files to XML if available 
# in all 32 configs and all 32 DefaultConfigs directories
CONVERT_BIN_TO_XML_PARAMS_SCRIPT=ConvertParamsToXml.sh
if [ -f ${CONVERT_BIN_TO_XML_PARAMS_SCRIPT} ] ; then
    /bin/sh ${CONVERT_BIN_TO_XML_PARAMS_SCRIPT}
    RET=$?
    if [ "0" -eq "${RET}" ] ; then
        echo "$(date): Binary parameter files converted successfully to XML" >>${UPDATE_LOGFILE}
    fi
    if [ "2" -eq "${RET}" ] ; then
        echo "$(date): Converting binary to XML parameter files failed" >>${UPDATE_LOGFILE}
        exit 13
    fi
fi


# CR 442535: - if QVGA FPGA is used set MaxDisparity to 1024 in XML parameter file
#            - do it if updated (current) version is < 2.40 (handled in script)
#            - do it for all working and default configuration directories (handled in script)
#            - do it behind converting parameter files from binary representation to XML
#            - update script is not terminated in error case
PATCH_MAX_DISPARITY_SCRIPT=PatchMaxDisparityForQvga.sh
if [ -f ${PATCH_MAX_DISPARITY_SCRIPT} ] ; then
    /bin/sh ${PATCH_MAX_DISPARITY_SCRIPT}
    RET=$?
    # 0 == OK, 1 == nothing to do, 2 == error
    if [ "0" -eq "${RET}" ] ; then
        echo "$(date): changing value for parameter MaxDisparity to 1024 successfully executed." >>${UPDATE_LOGFILE}
    elif [ "2" -eq "${RET}" ] ; then
        echo "$(date): executing script ${PATCH_MAX_DISPARITY_SCRIPT} failed (error code: ${RET}), see syslog for details!" >>${UPDATE_LOGFILE}
    fi
else
    echo "$(date): missing script ${PATCH_MAX_DISPARITY_SCRIPT} in update file!" >>${UPDATE_LOGFILE}
    logger -p user.warning "$(date): missing script ${PATCH_MAX_DISPARITY_SCRIPT} in update file!"
fi



### REQ PH_FEAT_Freiraumueberwachung_355 and CR 449984:
### delete all CZM reference images "czm_reference_image.data" in 
### - /var/Counter/configs/0...31/
### - /var/Counter/DefaultConfigs/0...31/
DEL_ALL_CZM_REF_IMAGES_SCRIPT=DeleteAllCzmRefImages.sh
if [ -f ${DEL_ALL_CZM_REF_IMAGES_SCRIPT} ] ; then
    /bin/sh ${DEL_ALL_CZM_REF_IMAGES_SCRIPT}
    RET=$?
    if [ "0" -ne "$RET" ] ; then
        echo "$(date): Deleting CZM reference image files failed!" >>${UPDATE_LOGFILE}
        logger -p user.warning "$(date): Deleting CZM reference image files failed!"
    else
        echo "$(date): All CZM reference image files successfully removed." >>${UPDATE_LOGFILE}
        logger -p user.notice "$(date): All CZM reference image files successfully removed."
    fi
else
    echo "$(date): Could not delete CZM reference images (script '${DEL_ALL_CZM_REF_IMAGES_SCRIPT}' not found)" >>${UPDATE_LOGFILE}
    logger -p user.notice "$(date): Could not delete CZM reference images (script '${DEL_ALL_CZM_REF_IMAGES_SCRIPT}' not found)"
fi


# Set U-Boot environment variables for booting new images:
${V} fw_setenv ${UBOOT_VAR_KERNEL_ADDR} ${NEW_KERNEL_ADDR} >/dev/null || exit 8
${V} fw_setenv ${UBOOT_VAR_RFS_ID} ${NEW_RFS_ID} >/dev/null || exit 8

# To ensure we start from flash next time:
fw_setenv bootcmd run bc_f >/dev/null

echo "$(date): Successfully updated kernel and rfs to version ${VERSION_NEW}" >>${UPDATE_LOGFILE}

# Delete list of total supported protocols
rm -f /var/Counter/log/prot-list.txt >/dev/null

### Possible post update scripts:
OFFSET=0
for script in ${CUSTOMIZING_SCRIPT} ${POST_UPDATE_SCRIPT} ; do
    OFFSET=`expr ${OFFSET} + 80`
    if [ -f ${script} ] ; then
        echo "$(date): Executing script ${script} ..." >>${UPDATE_LOGFILE}
        /bin/sh ${script}
        RET=$?
        if [ "0" -ne "${RET}" ] ; then
            echo "$(date): script ${script} returned FAILURE #${RET}" >>${UPDATE_LOGFILE}
            exit `expr ${RET} + ${OFFSET}`
        fi
        echo "$(date): script ${script} executed successfully" >>${UPDATE_LOGFILE}
    fi
done

# reset update hour meter
/bin/rm -f ${UPDATE_HOUR_METER_LOG} >/dev/null 2>&1 
/bin/echo "0" > ${UPDATE_HOUR_METER_LOG}

exit 0
